/*
 * A thread safe table of curves
 *
 * Copyright (c) 2003 by John D. Ramsdell. All rights reserved.
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// See the table header for more information.

#include <wx/wxprec.h>

#ifndef WX_PRECOMP
#include <wx/string.h>
#include <wx/list.h>
#include <wx/thread.h>
#endif

#include <wx/listimpl.cpp>
#include "curve.hpp"
#include "table.hpp"

WX_DEFINE_LIST(Curve_list);

Table::Table() : m_inserted(FALSE)
{
  m_list.DeleteContents(TRUE);
}

void Table::clear()
{
  m_critical_section.Enter();

  m_list.Clear();
  m_inserted = FALSE;

  m_critical_section.Leave();
}

void Table::add(Curve *curve)
{
  m_critical_section.Enter();

  // Delete a curve with the same caption
  for (Curve_list::Node *node = m_list.GetFirst();
       node;
       node = node->GetNext()) {
    Curve *data = node->GetData();
    if (curve->has_same_caption(data)) {
      m_list.DeleteNode(node);
      break;
    }
  }
  // Insert curve
  m_list.Insert(curve);
  m_inserted = FALSE;

  m_critical_section.Leave();
}

// Be sure to delete data inserted by this method.
void Table::insert_curves(Curve_list& list)
{
  m_critical_section.Enter();

  for (Curve_list::Node *node = m_list.GetFirst();
       node;
       node = node->GetNext()) {
    Curve *data = node->GetData();
    list.Insert(new Curve(*data));
  }
  m_inserted = TRUE;

  m_critical_section.Leave();
}

// Have the current curves been inserted into some list?
bool Table::have_curves_been_inserted()
{
  m_critical_section.Enter();

  bool inserted = m_inserted;

  m_critical_section.Leave();

  return inserted;
}
